package org.jastadd.ros2rag.compiler;

import beaver.Parser;
import org.jastadd.ros2rag.ast.*;
import org.jastadd.ros2rag.parser.Ros2RagParser;
import org.jastadd.ros2rag.scanner.Ros2RagScanner;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * Testing Ros2Rag without parser.
 *
 * @author rschoene - Initial contribution
 */
public class SimpleMain {
  public static void main(String[] args) {
    /*
    // as soon as the cache of isInSafetyZone is invalidated, update the value of Robot.ShouldUseLowSpeed with its value
    [always] update Robot.ShouldUseLowSpeed with isInSafetyZone() using transformation();

    // when a (new?) value for ShouldUseLowSpeed is set, send it over via mqtt
    [always] write Robot.ShouldUseLowSpeed;

    // when an update of pose is read via mqtt, then update current position
    [always] read Joint.CurrentPosition using PoseToPosition;

    // PBPose is a datatype defined in protobuf
    PoseToPosition: map PBPose to Position using
      pose.position.x += sqrt(.5 * size.x)
      MAP round(2)
      x = x / 100
      IGNORE_IF_SAME
    ;

    --- using generated methods ---
    Joint j;
    j.getCurrentPosition().connectTo("/robot/joint2/pos");

    RobotArm r;
    // this should not be required
    r.getShouldUseLowSpeed().addObserver(j.getCurrentPosition());
    r.getShouldUseLowSpeed().connectTo("/robot/config/speed");
     */
    Ros2Rag model = new Ros2Rag();
    Program program = parseProgram(Paths.get("src", "test", "resources", "Example.relast"));
    model.setProgram(program);

    MappingDefinition mappingDefinition = new MappingDefinition();
    mappingDefinition.setID("PoseToPosition");
    mappingDefinition.setFromType(makeMappingDefinitionType("PBPose"));
    mappingDefinition.setFromVariableName("x");
    mappingDefinition.setToType(makeMappingDefinitionType("Position"));
    mappingDefinition.setContent("      pose.position.x += sqrt(.5 * size.x)\n" +
        "      MAP round(2)\n" +
        "      x = x / 100\n" +
        "      IGNORE_IF_SAME\n" +
        "    ;");
    model.addMappingDefinition(mappingDefinition);

    ReadFromMqttDefinition readFromMqttDefinition = new ReadFromMqttDefinition();
    readFromMqttDefinition.setAlwaysApply(false);
    readFromMqttDefinition.setToken(TokenComponent.createRef("Joint.CurrentPosition"));
    readFromMqttDefinition.addMapping(mappingDefinition);
    model.addUpdateDefinition(readFromMqttDefinition);

    model.treeResolveAll();

    System.out.println(model.generateAspect("Model"));
  }

  private static MappingDefinitionType makeMappingDefinitionType(String type) {
    JavaMappingDefinitionType result = new JavaMappingDefinitionType();
    result.setType(new SimpleJavaTypeUse(type));
    return result;
  }

  private static Program parseProgram(Path path) {
    try (BufferedReader reader = Files.newBufferedReader(path)) {
      Ros2RagScanner scanner = new Ros2RagScanner(reader);
      Ros2RagParser parser = new Ros2RagParser();
      return (Program) parser.parse(scanner);
    } catch (IOException | Parser.Exception e) {
      e.printStackTrace();
    }
    return null;
  }
}
